-- -----------------------------------------------------------------------------

local ADDON_NAME, ns = ...
local WOEZ_D,WOEZ_O,WOEZ_F = ns.datas,ns.functions,ns.frames

-- -----------------------------------------------------------------------------
-- Debug Functions
-- -----------------------------------------------------------------------------

function WOEZ_O:DEBUG(what,more,bug) 
    if not WOEZ_D.DEBUG then return end 
    local colors = {['VAR']      = WOEZ_D.colors.yellow,
                    ['CALL']     = WOEZ_D.colors.magenta,
                    ['REGISTER'] = WOEZ_D.colors.red,
                    ['FIRE']     = WOEZ_D.colors.green,
                    ['OPTIONS']  = WOEZ_D.colors.blue,
                    ['TABLE']    = WOEZ_D.colors.pink }
    local color = (colors[what] or WOEZ_D.colors.yellow)
    print(WOEZ_D.PRINT.."|cff"..color..(more and tostring(more) or '> ').."|r"..(bug and ' '..tostring(bug) or '')) 
end
function WOEZ_O:DEBUG_T(name,tabl) 
    if not WOEZ_D.DEBUG then return end 
    local res = " "  
    for k,v in pairs(tabl) do 
        res = res..k.." |cff"..WOEZ_D.colors.pink.."*|r "
    end 
    WOEZ_O:DEBUG('TABLE',name,res)
end
function WOEZ_O:DEBUG_TV(name,tabl) 
    if not WOEZ_D.DEBUG then return end 
    local res = ""  
    for k,v in pairs(tabl) do 
        res = res..k.." > "..tostring(v).." |cff"..WOEZ_D.colors.pink.."***|r "
    end 
    WOEZ_O:DEBUG('TABLE',name,res)
end
function WOEZ_O:COUNT(tab)  
    local count = 0 
    for _ in pairs(tab) do 
        count = count + 1 
    end 
    return count 
end

-- -----------------------------------------------------------------------------
-- Create Frame
-- -----------------------------------------------------------------------------

function WOEZ_O:FRAM(name,type,where,opt,
                     z,x,y,yy,zz,
                     w,h)
    local obj = name            
    local stuff = _G[obj] or CreateFrame(type,obj,where,opt)
    if z then stuff:SetPoint(z,x,y,yy,zz) end
    if w and h then stuff:SetSize(w,h) end
    return stuff
end
function WOEZ_O:FADE(name,where,allpoint,f)
    local obj = name            
    local stuff = _G[obj] or where:CreateTexture(obj, "BACKGROUND")             
    if allpoint then stuff:SetAllPoints(true) end       
          stuff:SetColorTexture(0,0,0,f)
    return stuff
end
function WOEZ_O:ICON(name,where,ico,  
                     z,x,y,
                     w,h)
    local obj = name            
    local stuff = _G[obj] or where:CreateTexture(obj,"HIGH") 
    if z then stuff:SetPoint(z,x,y) end
          stuff:SetSize(w,h)      
          stuff:SetTexture(ico) 
    return stuff
end
function WOEZ_O:FONT(name,where,  
                     z,x,y,yy,zz,   
                     w,h,
                     font,justifyH)
    local obj = name            
    local stuff = _G[obj] or where:CreateFontString(obj,nil,font) 
    if z then stuff:SetPoint(z,x,y,yy,zz) end
    if w and h then stuff:SetSize(w,h) end
          stuff:SetJustifyH(justifyH) 
          stuff:SetJustifyV("CENTER") 
          stuff:SetWordWrap(false) 
    return stuff
end

-- -----------------------------------------------------------------------------
-- GET STATES
-- -----------------------------------------------------------------------------

function WOEZ_O:GET_GUILD() 
    WOEZ_O:DEBUG('CALL','GET_GUILD')
    WOEZ_D.InGuild = IsInGuild()
    if WOEZ_D.InGuild then
        local guild, _, _ = GetGuildInfo("player");
        WO_ez_DBC.guild = guild
    end
    WO_ez_DBC.guild_ranks = WO_ez_DBC.guild_ranks or {}
end

function WOEZ_O:GET_GUILD_TABLE() 
    WOEZ_O:DEBUG('VAR','GET_GUILD_TABLE')
	wipe(WOEZ_D.guildOnline)
	wipe(WOEZ_D.guildiesAl)
	wipe(WOEZ_D.guildRoles)
	wipe(WOEZ_D.Roles)

    WO_ez_DB.pvate_roles = WO_ez_DB.pvate_roles or {}
	local totalMembers = GetNumGuildMembers()
    for i = 1, totalMembers do
        local name, _, _, _, _, _, note, officernote, connected, _, _, _, _, _, _, _, guid = GetGuildRosterInfo(i)
        if not name then return end
        local PName = gsub(name,'%-'..WOEZ_D.playerrlm,'') 
        if connected then WOEZ_D.guildOnline[PName] = true end

        WOEZ_D.guildNotes[PName] = WO_ez_DBC.enable_noteo and officernote or note
        WOEZ_D.guildiesAl[PName] = true

        local role=nil

        local noteto = WO_ez_DBC.enable_icnof and officernote or note 
            if noteto:lower():find("tank") 
            or noteto:lower():find("tnk")  then role = "T"
        elseif noteto:lower():find("heal") then role = "H"
        elseif noteto:lower():find("dps") 
            or noteto:lower():find("melee") 
            or noteto:lower():find("cac")  then role = "M"
        elseif noteto:lower():find("range") 
            or noteto:lower():find("rng") 
            or noteto:lower():find("dist") then role = "R"
        end
        WOEZ_D.guildRoles[PName] = role

        if WO_ez_DB.pvate_roles[PName] then role = WO_ez_DB.pvate_roles[PName] end
        WOEZ_D.Roles[PName] = role
        
        local layer_name = "WOezRoster_"..PName 

        if UnitInParty(PName) or UnitInRaid(PName) then local objn=layer_name.."_conn" if _G[objn] then _G[objn]:SetVertexColor(0,.6,.9, 1) end
                  elseif WOEZ_D.guildOnline[PName] then local objn=layer_name.."_conn" if _G[objn] then _G[objn]:SetVertexColor(0,.9,.2, 1) end
                   elseif WOEZ_D.guildiesAl[PName] then local objn=layer_name.."_conn" if _G[objn] then _G[objn]:SetVertexColor(1, 1, 1,.3) end
                                                   else local objn=layer_name.."_conn" if _G[objn] then _G[objn]:SetVertexColor(1, 1, 1,.05) end end

        local objname = layer_name.."_note" if _G[objname] then _G[objname]:SetText(WOEZ_D.guildNotes[PName]) end
        local objname = layer_name.."_role" if _G[objname] then 
          if WOEZ_D.Roles[PName] and WOEZ_D.Roles[PName] ~= "E" then _G[objname]:SetTexture(WOEZ_D.FOLDR..WOEZ_D.Roles[PName]..".tga")
                                                                else _G[objname]:SetTexture(nil) end
        end
	end
end

-- -----------------------------------------------------------------------------
-- UPDATING
-- -----------------------------------------------------------------------------

function WOEZ_O:INVIT(who,add)
    if add and WOEZ_D.playernme ~= who then
        WOEZ_D.pToInvite[who] = true 
        WOEZ_D.pToRemove[who] = nil 
        WOEZ_D.toProcess = WOEZ_D.toProcess+1
    elseif WOEZ_D.playernme ~= who then
        WOEZ_D.pToInvite[who] = nil
        WOEZ_D.pToRemove[who] = true 
        WOEZ_D.toProcess = WOEZ_D.toProcess+1
    end
end
function WOEZ_O:INVITS(tablewho,add)
    WOEZ_O:DEBUG('CALL','INVITS')
    for who,_ in pairs(tablewho) do
        WOEZ_O:INVIT(who,add)
    end
end

function WOEZ_O:CLEAN_INVITS() 
    for k,_ in pairs(WOEZ_D.pToInvite) do 
        if WOEZ_D.pToInvi_p[k] or WOEZ_D.attendues[k] then 
            WOEZ_D.pToInvite[k]=nil 
        end 
    end 
    for k,_ in pairs(WOEZ_D.pToRemove) do 
        if not WOEZ_D.attendues[k] and not WOEZ_D.pToInvi_p[k] then 
            WOEZ_D.pToRemove[k]=nil 
        end 
    end 
    local NB_ToInvite,NB_ToRemove = WOEZ_O:COUNT(WOEZ_D.pToInvite),WOEZ_O:COUNT(WOEZ_D.pToRemove)
    WOEZ_D.toProcess = NB_ToInvite+NB_ToRemove
end

function WOEZ_O:DO_INVITS()
    WOEZ_O:DEBUG('FIRE','DO_INVITS',WOEZ_D.toProcess)
    if WOEZ_D.canEdit and WOEZ_D.eInfo and WOEZ_D.isOpen and WOEZ_D.eInfo.calendarType == "PLAYER" then

        WOEZ_O:CLEAN_INVITS()
        WOEZ_O:DEBUG_T('attendues',WOEZ_D.attendues) 
        WOEZ_O:DEBUG_T('pToInvite',WOEZ_D.pToInvite) 
        WOEZ_O:DEBUG_T('pToInvi_p',WOEZ_D.pToInvi_p) 
        WOEZ_O:DEBUG_T('pToRemove',WOEZ_D.pToRemove) 
        local howManyLeft = WOEZ_D.toProcess.." LEFT..."

        if WOEZ_D.toProcess == 0 then 
            WOEZ_O:PROGRESS("... Done !")
            WOEZ_O:PROGRESS_STOP() 
            WOEZ_O:REFRESH_ROSTER() 
        else
            ---------------------------------------------------------- to invite
            local toInvit, _ = next(WOEZ_D.pToInvite) 
            if toInvit ~= nil and not WOEZ_D.InPrgrss then 
                if WOEZ_D.eInfo.isLocked then
                    WOEZ_O:PROGRESS("|cff"..datas.colors.red.."[ "..WOEZ_D.eInfo.title.." ] locked...")
                    wipe(WOEZ_D.pToInvite)
                    wipe(WOEZ_D.pToInvi_p)
                    return
                else  
                    WOEZ_O:PROGRESS(howManyLeft.." |cff"..WOEZ_D.colors.yellow.."ADDING "..toInvit)
                    WOEZ_D.InPrgrss = true 
                    WOEZ_D.pToInvi_p[toInvit] = true
                    if not toInvit:find("-") then
                        C_Calendar.EventInvite(toInvit.."-"..WOEZ_D.playerrlm) 
                    else
                        C_Calendar.EventInvite(toInvit) 
                    end
                end
            else
                ------------------------------------------------------ to remove
                local toRemove, _ = next(WOEZ_D.pToRemove) 
                if toRemove ~= nil then
                    WOEZ_O:PROGRESS(howManyLeft.." |cff"..WOEZ_D.colors.yellow.."REMOVING "..toRemove)
                    local NumInv = C_Calendar.GetNumInvites()    
                    for i = 1, NumInv do
                        local inviteData = C_Calendar.EventGetInvite(i)
                        if toRemove == inviteData.name then 
                            C_Calendar.EventRemoveInvite(i) 
                            break
                        end
                    end
                end
                ----------------------------------------------------- end invite
            end
            WOEZ_O:REFRESH_ROSTER() 
        end 
    else
        WOEZ_O:PROGRESS_STOP() 
        WOEZ_O:REFRESH_ROSTER() 
    end

end

function WOEZ_O:PROGRESS(msg)
    -- WOEZ_O:DEBUG('CALL','PROGRESS')
    WOEZ_F.progress_t:SetText(msg or "UPDATING...")
    WOEZ_F.progress:Show()
end

function WOEZ_O:PROGRESS_STOP()
    WOEZ_D.pToRemove,WOEZ_D.pToInvite,WOEZ_D.pToInvi_p,WOEZ_D.toProcess,WOEZ_D.InPrgrss,WOEZ_D.toPrgrss = {}, {}, {}, 0, false, false
    WOEZ_F.progress:Hide()
end

function WOEZ_O:GICON(ico) 
    local ext = '.tga'
        if ico==0 then icon =   "onway"..ext  
    elseif ico==1 then icon = "nbNRply"..ext
    elseif ico==2 then icon = "nbCommg"..ext  
    elseif ico==3 then icon = "nbNCmmg"..ext
    elseif ico==4 then icon = "nbCommg"..ext  
    elseif ico==5 then icon = "nbNCmmg"..ext
    elseif ico==6 then icon = "nbNRply"..ext  
    elseif ico==7 then icon = "nbCommg"..ext
    elseif ico==8 then icon = "nbNRply"..ext  
    elseif ico==9 then icon = "nbNRply"..ext end    
    return WOEZ_D.FOLDR..icon
end

-- -----------------------------------------------------------------------------
-- CALENDAR / EVENT
-- -----------------------------------------------------------------------------

function WOEZ_O:GET_EVENT() 
    WOEZ_O:DEBUG('CALL','GET_EVENT')
    WOEZ_D.eInfo = C_Calendar.GetEventInfo()
    if WOEZ_D.eInfo then
        WOEZ_D.canEdit = C_Calendar.EventCanEdit()
    end
end

function WOEZ_O:GET_EVENT_INVITS() 
    if WOEZ_D.eInfo and WOEZ_D.isOpen then
        WOEZ_O:DEBUG('CALL','GET_EVENT_INVITS')

        wipe(WOEZ_D.attendues) 
        wipe(WOEZ_D.OutofRostr) 
        local Invites, Comming, NoReply, NotCmmg, WatCmmg, nbTNK, nbHLS, nbMLE, nbRNG = 0,0,0,0,0,0,0,0,0
        local t,h,m,r = 0,0,0,0
        local NumInv = C_Calendar.GetNumInvites()
        for i = 1, NumInv do
            local inviteData = C_Calendar.EventGetInvite(i)
            local inviteName = inviteData.name
            if inviteName ~= nil then 

                if not WOEZ_D.guildiesAl[inviteName] then 
                    WOEZ_D.OutofRostr[inviteName] = { lvl=inviteData.level, class=inviteData.classFilename } 
                end

                WOEZ_D.attendues[inviteName] = {index=i,inviteStatus=inviteData.inviteStatus} 
                WOEZ_D.pToInvite[inviteName] = nil
                WOEZ_D.pToInvi_p[inviteName] = nil
                Invites = Invites+1
                    if inviteData.inviteStatus==1 then NoReply=NoReply+1 -- CALENDAR_INVITESTATUS_INVITED
                elseif inviteData.inviteStatus==2 then Comming=Comming+1 -- CALENDAR_INVITESTATUS_ACCEPTED
                elseif inviteData.inviteStatus==3 then NotCmmg=NotCmmg+1 -- CALENDAR_INVITESTATUS_DECLINED
                elseif inviteData.inviteStatus==4 then Comming=Comming+1 -- CALENDAR_INVITESTATUS_CONFIRMED
                elseif inviteData.inviteStatus==5 then NotCmmg=NotCmmg+1 -- CALENDAR_INVITESTATUS_OUT
                elseif inviteData.inviteStatus==6 then WatCmmg=WatCmmg+1 -- CALENDAR_INVITESTATUS_STANDBY
                elseif inviteData.inviteStatus==7 then Comming=Comming+1 -- CALENDAR_INVITESTATUS_SIGNEDUP
                elseif inviteData.inviteStatus==8 then NoReply=NoReply+1 -- CALENDAR_INVITESTATUS_NOT_SIGNEDUP
                elseif inviteData.inviteStatus==9 then WatCmmg=WatCmmg+1 -- CALENDAR_INVITESTATUS_TENTATIVE
                end
                if inviteData.inviteStatus==2 or inviteData.inviteStatus==4 or inviteData.inviteStatus==7 then
                        if WOEZ_D.Roles[inviteName] == "T" then t=t+1
                    elseif WOEZ_D.Roles[inviteName] == "H" then h=h+1
                    elseif WOEZ_D.Roles[inviteName] == "M" then m=m+1
                    elseif WOEZ_D.Roles[inviteName] == "R" then r=r+1 end
                end
            end
        end
        NoReply = (WatCmmg > 0 and NoReply.." + "..WatCmmg or NoReply)
        
        local infospos = WOEZ_D.sizes.MARGES
        -- not comming
        WOEZ_F.infos_NCmmg_t:SetText(format('|cff'..WOEZ_D.colors.red    ..'%s',NotCmmg))
        -- no reply
        infospos = infospos+WOEZ_D.sizes.infos_NCmmg_i+WOEZ_F.infos_NCmmg_t:GetStringWidth() +WOEZ_D.sizes.MARGES-4
        WOEZ_F.infos_NRply_i:SetPoint("RIGHT",-(infospos),0) -- icon pos      
        WOEZ_F.infos_NRply_t:SetText(format('|cff'..WOEZ_D.colors.grey   ..'%s',NoReply))          
        -- comming
        infospos = infospos+WOEZ_D.sizes.infos_NRply_i+WOEZ_F.infos_NRply_t:GetStringWidth() +WOEZ_D.sizes.MARGES-2
        WOEZ_F.infos_Commg_i:SetPoint("RIGHT",-(infospos),0)
        WOEZ_F.infos_Commg_t:SetText(format('|cff'..WOEZ_D.colors.yellow ..'%s',Comming))
        -- invits
        WOEZ_F.infos_Invit_t:SetText(format('|cff'..WOEZ_D.colors.white  ..'%s INVITS',Invites))
        -- resume
        WOEZ_F.sum_rangs_t:SetText(format('|cff'..WOEZ_D.colors.ranges..'%s',r)) 
        WOEZ_F.sum_meles_t:SetText(format('|cff'..WOEZ_D.colors.melees..'%s',m)) 
        WOEZ_F.sum_heals_t:SetText(format('|cff'..WOEZ_D.colors.heals.. '%s',h)) 
        WOEZ_F.sum_tanks_t:SetText(format('|cff'..WOEZ_D.colors.tanks.. '%s',t)) 

    end
    
end

-- -----------------------------------------------------------------------------
-- ROSTER VIEW
-- -----------------------------------------------------------------------------

function WOEZ_O:CLOSE_ROSTER() 
    WOEZ_O:DEBUG('CALL','CLOSE_ROSTER')
    WOEZ_D.isOpen = false
    WOEZ_O:PROGRESS_STOP()
    WOEZ_F.main:Hide()
    WOEZ_F.main:ClearAllPoints()
end

function WOEZ_O:REFRESH_ROSTER() 
    if WOEZ_D.eInfo and WOEZ_D.isOpen then
        WOEZ_O:DEBUG('CALL','REFRESH_ROSTER ??')
        WOEZ_O:GET_EVENT_INVITS() 
        WOEZ_O:DEBUG_TV('OutofRostr',WOEZ_D.OutofRostr)

        if _G["WOezRank_0"] then
            local all_OutofRostr = { _G["WOezRank_0"]:GetChildren() }
            for _, OutofRosxr in ipairs(all_OutofRostr) do 
                if OutofRosxr:GetObjectType() == "CheckButton" then OutofRosxr:Hide() end
            end
        end

        local k = 0
        for name,infos in pairs(WOEZ_D.OutofRostr) do
            k = k+1
            local nplayer, nrealm = strsplit( "-", name, 2 )
            WOEZ_O:SET_LINE_play(0,k,_G["WOezRank_0"],name,RAID_CLASS_COLORS[infos.class],infos.lvl,"REALM: "..(nrealm or WOEZ_D.playerrlm)) 
        end

        if WO_ez_DBC.only_invited then 
                WOEZ_F.bottnc:SetChecked(true)     WOEZ_F.bottxt:SetTextColor(1,1,1,1) 
        else    WOEZ_F.bottnc:SetChecked(false)    WOEZ_F.bottxt:SetTextColor(.4,.4,.4,1)         end

        local hei, hei_rank,hei_pv = WOEZ_D.fonts.TTL_07, 0, 0
        local all_ranks = { WOEZ_F.child:GetChildren() }
        for _, rank in ipairs(all_ranks) do 
            if rank:GetObjectType() == "CheckButton" then 
                ----------------------------------------------------------------
                ----------------------------------------------------------------
                local rankName = rank:GetName() 
                local RName      = _G[rankName.."_txt"]:GetText() 
                if RName then
                    local toadd = {}
                    rank:Show()
                    local rankIDn  = rankName:gsub("WOezRank_", "",1) 
                    local rankID   = tonumber(rankIDn)

                    if rankID == 0 and WO_ez_DBC.pvate_totop then
                        rank:SetPoint("TOPLEFT",WOEZ_D.fonts.TXT_05,"-"..hei)
                    else
                        rank:SetPoint("TOPLEFT",WOEZ_D.fonts.TXT_05,"-"..hei)
                    end

                    if WO_ez_DBC.only_invited then 
                        rank:SetAlpha(1) 
                        _G[rankName.."_txt"]:SetAlpha(.4)       
                        _G[rankName.."_btn"]:Hide()    
                        
                        hei_rank = -(WOEZ_D.sizes.lines_h)    
                        rank:SetEnabled(false)                                                    
                    else 
                        rank:SetAlpha(WO_ez_DBC.guild_ranks[rankID] and .4 or 1) 
                        _G[rankName.."_txt"]:SetAlpha(1)                              
                        _G[rankName.."_btn"]:Show()     

                        hei,hei_rank = hei+WOEZ_D.sizes.lines_h,0  
                        
                        if WOEZ_D.canEdit and not WOEZ_D.eInfo.isLocked and WOEZ_D.eInfo.calendarType == "PLAYER" then rank:SetEnabled(true) else rank:SetEnabled(false) end      
                    end

                    local toadd_nbCheck,toadd_nb = 0,0  

                    local all_players = { rank:GetChildren() }
                    for _, player in ipairs(all_players) do 
                        if player:GetObjectType() == "CheckButton" then 
                            if rankID == 0 and not WO_ez_DBC.pvate_totop then
                                hei_pv,hei_rank,toadd_nb,toadd_nbCheck,ntoadd = WOEZ_O:REFRESH_LINE_play(player,rankID,hei_pv,hei_rank,toadd_nb,toadd_nbCheck)
                            else
                                hei,hei_rank,toadd_nb,toadd_nbCheck,ntoadd = WOEZ_O:REFRESH_LINE_play(player,rankID,hei,hei_rank,toadd_nb,toadd_nbCheck)
                            end
                            if ntoadd then toadd[ntoadd] = true end
                        end
                    end

                    rank:SetScript("OnClick", function(self, button)
                        WOEZ_O:DEBUG_T('toadd',toadd)
                        self:SetEnabled(false)
                        if self:GetChecked() then 
                            WOEZ_O:INVITS(toadd,true) 
                            local all_players = { rank:GetChildren() }
                            for _, player in ipairs(all_players) do 
                                if player:GetObjectType() == "CheckButton" then 
                                    player:SetEnabled(false) 
                                    player:SetChecked(true) 
                                end
                            end
                        else 
                            WOEZ_O:INVITS(toadd,false) 
                            local all_players = { rank:GetChildren() }
                            for _, player in ipairs(all_players) do 
                                if player:GetObjectType() == "CheckButton" then 
                                    player:SetEnabled(false) 
                                    player:SetChecked(false) 
                                end
                            end
                        end
                    end)  

                    if WO_ez_DBC.only_invited then
                        if toadd_nbCheck > 0 then 
                            rank:Show() 
                            _G[rankName.."_txt"]:Hide()       
                            _G[rankName.."_btn"]:Hide()   
                        else 
                            rank:Hide() 
                            _G[rankName.."_txt"]:Show()       
                            _G[rankName.."_btn"]:Show()   
                        end
                    else
                        if toadd_nb > 0 then 
                            rank:Show() 
                            if rankID == 0 and not WO_ez_DBC.pvate_totop then
                                hei = hei-WOEZ_D.sizes.lines_h
                            else
                                hei = hei+WOEZ_D.sizes.MMARGS
                            end
                        else 
                            
                            if not WO_ez_DBC.pvate_totop then 
                                hei_pv = hei_pv-WOEZ_D.sizes.lines_h-WOEZ_D.sizes.MMARGS
                            end
                            hei = hei-WOEZ_D.sizes.lines_h
                            rank:Hide() 
                        end
                        _G[rankName.."_txt"]:Show()       
                        _G[rankName.."_btn"]:Show()   
                    end
                    if toadd_nbCheck == toadd_nb or WO_ez_DBC.only_invited then rank:SetChecked(true) else rank:SetChecked(false) end
                else
                    rank:Hide()
                ----------------------------------------------------------------
                end ------------------------------------------------------------
            end
        end
        if not WO_ez_DBC.pvate_totop then 
            local rank = _G["WOezRank_0"]
            rank:SetPoint("TOPLEFT",WOEZ_D.fonts.TXT_05,"-"..hei)
            hei = hei+hei_pv+WOEZ_D.sizes.lines_h+WOEZ_D.sizes.MMARGS
        end
        if WO_ez_DBC.only_invited then
            if not WO_ez_DBC.pvate_totop then 
                hei = hei-WOEZ_D.sizes.lines_h
            else
                hei = hei+WOEZ_D.sizes.MMARGS
            end
        end

        if WO_ez_DBC.enable_notes then WOEZ_F.txtwidth:SetWidth(WOEZ_D.play_txt_maxwidth+WOEZ_D.sizes.MARGES*1.5+WOEZ_D.fonts.TXT_05+WOEZ_D.fonts.TXT_14)
                                  else WOEZ_F.txtwidth:SetWidth(WOEZ_D.play_txt_maxwidth+WOEZ_D.sizes.MARGES*3  +WOEZ_D.fonts.TXT_05+WOEZ_D.fonts.TXT_14) end
        if WO_ez_DBC.enable_ilevl then WOEZ_F.txtlevel:SetWidth(WOEZ_D.play_lvl_maxwidth+WOEZ_D.sizes.MMARGS) 
                                  else WOEZ_F.txtlevel:SetWidth(0.1) end

        local Phei = (WOEZ_F.main:GetHeight() - WOEZ_D.sizes.MARGES*3 - WOEZ_D.sizes.PANNELINFOS_H - WOEZ_D.sizes.PANNELONLYC_H)
        local max_hei = (hei-Phei<1 and 1 or hei-Phei)
        WOEZ_F.scrollb:SetMinMaxValues(1, max_hei)
    end
end

function WOEZ_O:SET_ROSTER() 
    WOEZ_O:DEBUG('CALL','--->','SET_ROSTER')
    WOEZ_O:PROGRESS_STOP()
    if WOEZ_D.eInfo and WOEZ_D.isOpen and _G.CalendarFrame and _G.CalendarFrame:IsShown() and (WOEZ_D.eInfo.calendarType == "PLAYER" or WOEZ_D.eInfo.calendarType == "GUILD_EVENT") then

        WOEZ_O:DEBUG('CALL','---> SET_ROSTER','OK')
        GuildRoster()        
        ------------------------------------------------------------------------
        ------------------------------------------------------------------ EMPTY
        local all_ranks = { WOEZ_F.child:GetChildren() }
        for _, rank in ipairs(all_ranks) do 
            if rank:GetObjectType() == "CheckButton" then 
                local rankName = rank:GetName() 
                   _G[rankName.."_txt"]:SetText('')  
                local all_players = { rank:GetChildren() }
                for _, player in ipairs(all_players) do 
                    if player:GetObjectType() == "CheckButton" then 
                        local playerName = player:GetName() 
                           _G[playerName.."_txt"]:SetText('')
                    end
                end
            end
        end
        ------------------------------------------------------------------------
        ----------------------------------------------------------- out of guild
        local rank = WOEZ_O:SET_LINE_rank(0,WO_ez_DBC.pvate_name) 
        ------------------------------------------------------------------------
        ------------------------------------------------------------ is in guild 
        if WOEZ_D.InGuild then
            WOEZ_O:GET_GUILD_TABLE()
            --------------------------------------------------- get guild roster
            local RosterGuild = {}
            local numGuildMembers, numOnline, numOnlineAndMobile = GetNumGuildMembers()
            for z=1,numGuildMembers do
                local name, _, rankIndex, level, _, _, note, officernote, _, _, classFileName,  _, _, _ = GetGuildRosterInfo(z);
                local rang = (rankIndex == 0 and (WO_ez_DBC.guildrankGM or 2) or rankIndex+1)
                local noteto = WO_ez_DBC.enable_icnof and officernote or note 
                RosterGuild[rang] = RosterGuild[rang] or {}
                RosterGuild[rang][z] = {Nname=gsub(name,'%-'..WOEZ_D.playerrlm,''),lvl=level,note=noteto,classcolor=RAID_CLASS_COLORS[classFileName]}
            end
            --------------------------------------------------------------------
            --------------------------------------------------------------------
            for i=2, GuildControlGetNumRanks() do 
                local NameRank = GuildControlGetRankName(i)

                if RosterGuild[i] ~= nil and NameRank ~= nil then -- members rank
                    -- ---------------------------------------------------------
                    local rank = WOEZ_O:SET_LINE_rank(i,NameRank) 
                    -- ---------------------------------------------------------
                    for k,c in pairs(RosterGuild[i]) do
                        local color     = c.classcolor  
                        local alpha     = 1
                        if WO_ez_DBC.enable_ilevf then alpha = (c.lvl < WOEZ_D.playerlvl and .3 or 1) end
                        local play,toa = WOEZ_O:SET_LINE_play(i,k,rank,c.Nname,color,c.lvl,c.note) 
                    end 
                    -- ---------------------------------------------------------
                    -- ---------------------------------------------------------
                    
                end -- end / if RosterGuild[i] ~= nil and NameRank ~= nil then
                    
            end -- end / for i=2, GuildControlGetNumRanks() do

        end -- end / if WOEZ_D.InGuild then ------------------------------------
        ------------------------------------------------------------------------

        ------------------------------------------------------------- sizes cols
        if WO_ez_DBC.enable_notes then WOEZ_F.txtwidth:SetWidth(WOEZ_D.play_txt_maxwidth+WOEZ_D.sizes.MARGES*1.5+WOEZ_D.fonts.TXT_05+WOEZ_D.fonts.TXT_14)
                                  else WOEZ_F.txtwidth:SetWidth(WOEZ_D.play_txt_maxwidth+WOEZ_D.sizes.MARGES*3  +WOEZ_D.fonts.TXT_05+WOEZ_D.fonts.TXT_14) end
        if WO_ez_DBC.enable_ilevl then WOEZ_F.txtlevel:SetWidth(WOEZ_D.play_lvl_maxwidth+WOEZ_D.sizes.MMARGS) 
                                  else WOEZ_F.txtlevel:SetWidth(0.1) end
        ------------------------------------------------------------------------
        WOEZ_F.main:Show()
        WOEZ_O:REFRESH_ROSTER() 
    else 
        WOEZ_O:DEBUG('CALL','---> SET_ROSTER','NO')
        WOEZ_O:CLOSE_ROSTER() 
    end
end

function WOEZ_O:SET_LINE_rank(i,name) 
    local rank = WOEZ_O:FRAM("WOezRank_"..i,"CheckButton",WOEZ_F.child,"ChatConfigCheckButtonTemplate",nil,nil,nil,nil,nil,WOEZ_D.fonts.TXT_14,WOEZ_D.fonts.TXT_14)
    rank:Show()
    -- text
    local rank_txt = WOEZ_O:FONT("WOezRank_"..i.."_txt",rank,"LEFT",rank,"RIGHT",WOEZ_D.sizes.MMARGS,0, nil,nil,WOEZ_D.fonts.TTL_F ,"LEFT")
    rank_txt:SetPoint("RIGHT",WOEZ_F.scrollf, "RIGHT",-(WOEZ_D.sizes.MARGES),0)
    rank_txt:SetText(string.upper(name)..WOEZ_D.texte.points)   
    
    rank_txt:SetTextColor(1,1,1,1) 
    -- expand
    local rank_btn = WOEZ_O:FRAM("WOezRank_"..i.."_btn","Button",rank,"UIPanelButtonTemplate")                    
    rank_btn:SetPoint("RIGHT",WOEZ_F.scrollf,"RIGHT",-(WOEZ_D.sizes.MARGES),0)
    rank_btn:SetPoint("LEFT" ,rank          ,"RIGHT",WOEZ_D.sizes.MARGES   ,0)
    rank_btn:SetScript("OnClick", function() 
          if WO_ez_DBC.guild_ranks[i] then 
             WO_ez_DBC.guild_ranks[i] = nil 
        else WO_ez_DBC.guild_ranks[i] = true end
        WOEZ_O:REFRESH_ROSTER() 
    end)
    rank_btn:SetAlpha(0) 
    rank_btn:SetFrameStrata("HIGH") 
    return rank
end
function WOEZ_O:SET_LINE_play(i,k,rank,name,color,lvl,note) 
    local layer_name = "WOezRoster_"..name 

    local play      = WOEZ_O:FRAM(layer_name,"CheckButton",rank,"ChatConfigCheckButtonTemplate","TOPLEFT",0,0,nil,nil,WOEZ_D.fonts.TXT_14,WOEZ_D.fonts.TXT_14)
    local play_txt  = WOEZ_O:FONT(layer_name.."_txt",play,"LEFT",play,"RIGHT",WOEZ_D.sizes.MMARGS,0, nil,nil,WOEZ_D.fonts.TXT_F ,"LEFT")
          play_txt:SetPoint("RIGHT",WOEZ_F.txtwidth, "RIGHT",0,0)

    local nplayer, nrealm = strsplit( "-", name, 2 )
          play_txt:SetText(nplayer)   
          play_txt:SetTextColor(color.r, color.g, color.b,alpha) 

    local play_full = WOEZ_O:FONT(layer_name.."_full",play,nil,nil,nil,nil,nil,nil,nil, WOEZ_D.fonts.TXT_F,"LEFT")
          play_full:SetText(name)  
          play_full:Hide()
           
    local play_lvl  = WOEZ_O:FONT(layer_name.."_lvl",play,"LEFT",play_txt,"RIGHT",WOEZ_D.sizes.MMARGS,0,nil,nil, WOEZ_D.fonts.TXT_F ,"RIGHT")
          play_lvl:SetPoint("RIGHT",WOEZ_F.txtlevel,"RIGHT",0,0)
          play_lvl:SetText(lvl) 
                            
    local play_icn  = WOEZ_O:ICON(layer_name.."_icn",play,nil,nil,nil,nil,WOEZ_D.fonts.TXT_12-WOEZ_D.sizes.MMARGS,WOEZ_D.fonts.TXT_12-WOEZ_D.sizes.MMARGS)
          play_icn:SetPoint("RIGHT",play_txt,"RIGHT",WOEZ_D.sizes.MMARGS,0)

    local play_nte  = WOEZ_O:FONT(layer_name.."_note",play,"LEFT",play_lvl,"RIGHT",WOEZ_D.sizes.MMARGS,0, nil,nil,WOEZ_D.fonts.TXT_F ,"LEFT")
          play_nte:SetText(note)   
    
    local play_bbl  = WOEZ_O:ICON(layer_name.."_conn",play,WOEZ_D.FOLDR .."round.tga",nil,nil,nil,        WOEZ_D.fonts.TXT,WOEZ_D.fonts.TXT)                        
    local play_rol  = WOEZ_O:ICON(layer_name.."_role",play,nil,"RIGHT" ,play_bbl,"LEFT",WOEZ_D.fonts.TXT_12,WOEZ_D.fonts.TXT_12)

    if WO_ez_DBC.enable_ilevl then play_lvl:SetTextColor(1,1,1,alpha) 
                              else play_lvl:SetTextColor(1,1,1,0)  end
    if WO_ez_DBC.enable_notes then play_nte:SetTextColor(.4,.4,.4,alpha) 
                              else play_nte:SetTextColor(.4,.4,.4,0)  end

    local play_rbt = WOEZ_O:FRAM(layer_name.."_rbtn","Button",play,"UIPanelButtonTemplate")                    
          play_rbt:SetPoint("TOPRIGHT",play_rol,"TOPRIGHT",0,2)
          play_rbt:SetPoint("BOTTOMLEFT" ,play_rol,"BOTTOMLEFT",0,-2)
          play_rbt:SetScript("OnClick", function() 

            local oldrole = WO_ez_DB.pvate_roles[name] and WO_ez_DB.pvate_roles[name] or WOEZ_D.Roles[name] and WOEZ_D.Roles[name] or nil
            local newrole = nil
            if oldrole == nil or oldrole == "E" then newrole = "M"
            elseif oldrole == "M" then newrole = "R"
            elseif oldrole == "R" then newrole = "H"
            elseif oldrole == "H" then newrole = "T"
            elseif oldrole == "T" then newrole = "E" end
            WO_ez_DB.pvate_roles[name] = newrole 
            WOEZ_D.Roles[name] = newrole

            if newrole == WOEZ_D.guildRoles[name] or (not WOEZ_D.guildRoles[name] and newrole == "E") then  
                WO_ez_DB.pvate_roles[name] = nil 
            end
            local objname = "WOezRoster_"..name.."_role" if _G[objname] then 
                if newrole ~= nil and newrole ~= "E" then _G[objname]:SetTexture(WOEZ_D.FOLDR..newrole..".tga")
                                                     else _G[objname]:SetTexture(nil) end
                WOEZ_O:GET_EVENT_INVITS() 
            end

          end)
          play_rbt:SetAlpha(0) 
          play_rbt:SetFrameStrata("HIGH") 

    if WO_ez_DBC.enable_icons then
          play_rol:Show() play_rbt:Show()
          play_nte:SetPoint("RIGHT",WOEZ_F.scrollf, "RIGHT",-(WOEZ_D.sizes.MARGES*2.5+WOEZ_D.fonts.TXT_12),0)
          play_bbl:SetPoint("RIGHT",play_nte,"RIGHT",(WOEZ_D.sizes.MARGES*1.7)+WOEZ_D.fonts.TXT_12,0)
    else  play_rol:Hide() play_rbt:Hide()
          play_nte:SetPoint("RIGHT",WOEZ_F.scrollf, "RIGHT",-(WOEZ_D.sizes.MARGES*2.5),0)
          play_bbl:SetPoint("RIGHT",play_nte,"RIGHT",(WOEZ_D.sizes.MARGES*1.7),0)  
    end

    local play_txt_w = play_txt:GetStringWidth() if play_txt_w > WOEZ_D.play_txt_maxwidth then WOEZ_D.play_txt_maxwidth = play_txt_w end    
    local play_lvl_w = play_lvl:GetStringWidth() if play_lvl_w > WOEZ_D.play_lvl_maxwidth then WOEZ_D.play_lvl_maxwidth = play_lvl_w end        
    
    play:SetScript("OnClick", function(self, button)
        play_icn:SetTexture(WOEZ_O:GICON(0)) 
        self:SetEnabled(false)
        if self:GetChecked() then WOEZ_O:INVIT(name,true) else WOEZ_O:INVIT(name,false) end
    end)  

    return play,name

end

function WOEZ_O:REFRESH_LINE_play(player,rankID,hei,hei_rank,toadd_nb,toadd_nbCheck) 
    local layer_name = player:GetName() 
    local checkname  = _G[layer_name.."_txt"]:GetText()
    local PName      = _G[layer_name.."_full"]:GetText() 

    if checkname then
        local isAttendue = WOEZ_D.attendues[PName] or false
        if WO_ez_DBC.only_invited then                  
            if isAttendue then
                hei_rank = hei_rank+WOEZ_D.sizes.lines_h
                hei = hei+WOEZ_D.sizes.lines_h
                player:Show()
            else      
                player:Hide()
            end
        else
            if not WO_ez_DBC.guild_ranks[rankID] then    
                hei_rank = hei_rank+WOEZ_D.sizes.lines_h                    
                hei = hei+WOEZ_D.sizes.lines_h
                player:Show()
            else      
                player:Hide()
            end
        end
        toadd_nb = toadd_nb+1
        player:SetPoint("TOPLEFT",0,-(hei_rank))

        if WOEZ_D.pToRemove[PName] then 
            _G[layer_name.."_icn"]:SetTexture(WOEZ_O:GICON(0))                    
            player:SetChecked(false)   
        elseif WOEZ_D.pToInvite[PName] or WOEZ_D.pToInvi_p[PName] then 
            _G[layer_name.."_icn"]:SetTexture(WOEZ_O:GICON(0))                          
            player:SetChecked(true)    
            toadd_nbCheck = toadd_nbCheck+1 
        elseif isAttendue then 
            _G[layer_name.."_icn"]:SetTexture(WOEZ_O:GICON(isAttendue.inviteStatus))   
            player:SetChecked(true)    
            toadd_nbCheck = toadd_nbCheck+1 
        else
            _G[layer_name.."_icn"]:SetTexture(nil)                   
            player:SetChecked(false)   
        end

        if WOEZ_D.canEdit and not WOEZ_D.eInfo.isLocked and WOEZ_D.eInfo.calendarType == "PLAYER" and playerName ~= WOEZ_D.playernme then player:SetEnabled(true) else player:SetEnabled(false) end
        if WO_ez_DBC.only_invited then player:SetEnabled(false) end
 
        if UnitInParty(PName) or UnitInRaid(PName) then local objn=layer_name.."_conn" if _G[objn] then _G[objn]:SetVertexColor(0,.6,.9, 1) end
                  elseif WOEZ_D.guildOnline[PName] then local objn=layer_name.."_conn" if _G[objn] then _G[objn]:SetVertexColor(0,.9,.2, 1) end
                   elseif WOEZ_D.guildiesAl[PName] then local objn=layer_name.."_conn" if _G[objn] then _G[objn]:SetVertexColor(1, 1, 1,.3) end
                                                   else local objn=layer_name.."_conn" if _G[objn] then _G[objn]:SetVertexColor(1, 1, 1,.05) end end

        if WO_ez_DB.pvate_roles[PName] then WOEZ_D.Roles[PName] = WO_ez_DB.pvate_roles[PName] end
        if WOEZ_D.Roles[PName] and WOEZ_D.Roles[PName] ~= "E" then local objn=layer_name.."_role" if _G[objn] then _G[objn]:SetTexture(WOEZ_D.FOLDR..WOEZ_D.Roles[PName]..".tga") end
                                                              else local objn=layer_name.."_role" if _G[objn] then _G[objn]:SetTexture(nil) end end
        if WOEZ_D.guildNotes[PName] then local objn=layer_name.."_note" if _G[objn] then _G[objn]:SetText(WOEZ_D.guildNotes[PName]) end end

        return hei,hei_rank,toadd_nb,toadd_nbCheck,PName
    else
        player:Hide()
        return hei,hei_rank,toadd_nb,toadd_nbCheck,nil
    end
end
-- -----------------------------------------------------------------------------
-- Addons
-- -----------------------------------------------------------------------------

function WOEZ_O:SHOW_WELCOME()  
    WOEZ_O:DEBUG('CALL','SHOW_WELCOME')
    
    WOEZ_O:DEBUG_TV('Local DB',WO_ez_DBC)

    local d_global = {
        pvate_roles = {},
    }
    local defaults = {
        width = 200,

        guild = "",
        guild_ranks = {},
        guildrankGM = 2,

        pvate_list = true,
        pvate_name = "Private list",
        pvate_roster = {},
        pvate_totop = true,

        enable_icons = true,
        enable_icnof = false,
        enable_ilevl = true,
        enable_ilevf = true,
        enable_notes = true,
        enable_noteo = false,
        enable_summa = true,
        only_invited = false,
    }

	local function copyDefaults(src, dst)
		if type(src) ~= "table" then return {} end
		if not type(dst) then dst = {} end
		for k, v in pairs(src) do
			if type(v) == "table" then
				dst[k] = copyDefaults(v, dst[k])
			elseif type(v) ~= type(dst[k]) then
				dst[k] = v
			end
		end
		return dst
	end


    WO_ez_DB  = copyDefaults(d_global, WO_ez_DB )
    WO_ez_DBC = copyDefaults(defaults, WO_ez_DBC)

    WOEZ_D.sizes.PANNEL = WO_ez_DBC.width
    WOEZ_O:DEBUG_TV('Local DB',WO_ez_DBC)    
    WOEZ_F.main:SetWidth(WOEZ_D.sizes.PANNEL)


end